# Functions for managing a domain's Unix user

$feature_depends{'unix'} = [ 'dir' ];

# setup_unix(&domain)
# Creates the Unix user and group for a domain
sub setup_unix
{
local $tmpl = &get_template($_[0]->{'template'});
&require_useradmin();
local (%uinfo, %ginfo);

# Create the group
&$first_print(&text('setup_group', $_[0]->{'group'}));
&useradmin::lock_user_files();
%ginfo = ( 'group', $_[0]->{'group'},
	   'gid', $_[0]->{'gid'} );
&useradmin::set_group_envs(\%ginfo, 'CREATE_GROUP');
&useradmin::making_changes();
&useradmin::create_group(\%ginfo);
&useradmin::made_changes();
if (!defined(getgrnam($_[0]->{'group'}))) {
	&second_print($text{'setup_ecrgroup'});
	exit;
	}
&$second_print($text{'setup_done'});

# Then the user
&$first_print(&text('setup_user', $_[0]->{'user'}));
%uinfo = ( 'user', $_[0]->{'user'},
	   'uid', $_[0]->{'uid'},
	   'gid', $_[0]->{'ugid'},
	   'pass', &useradmin::encrypt_password($_[0]->{'pass'}),
	   'real', $_[0]->{'owner'},
	   'home', $_[0]->{'home'},
	   'shell', $config{'unix_shell'} );
&set_pass_change(\%user);
&useradmin::set_user_envs(\%uinfo, 'CREATE_USER', $_[0]->{'pass'}, [ ]);
&useradmin::making_changes();
&useradmin::create_user(\%uinfo);
&useradmin::made_changes();
&useradmin::unlock_user_files();
if (!defined(getpwnam($_[0]->{'user'}))) {
	&$second_print($text{'setup_ecruser'});
	exit;
	}
&$second_print($text{'setup_done'});

# Set the user's quota
if ($config{'home_quotas'}) {
	&set_server_quotas($_[0]);
	}

&create_mail_file(\%user);
}

# modify_unix(&domain, &olddomain)
# Change the password and real name for a domain unix user
sub modify_unix
{
if (!$_[0]->{'parent'}) {
	&require_useradmin();
	local @allusers = &list_domain_users($_[1]);
	local ($uinfo) = grep { $_->{'user'} eq $_[1]->{'user'} } @allusers;
	if ($uinfo) {
		local %old = %$uinfo;
		&$first_print($text{'save_user'});
		$uinfo->{'real'} = $_[0]->{'owner'};
		if ($_[0]->{'pass_set'}) {
			# Update the Unix user's password
			local $enc = &useradmin::encrypt_password(
					$_[0]->{'pass'});
			if ($d->{'disabled'}) {
				# Just keep for later use when enabling
				$d->{'disabled_oldpass'} = $enc;
				}
			else {
				# Set password now
				$uinfo->{'pass'} = $enc;
				}
			$uinfo->{'plainpass'} = $_[0]->{'pass'};
			&set_pass_change($uinfo);
			}

		if ($_[0]->{'user'} ne $_[1]->{'user'}) {
			# Unix user was re-named
			$uinfo->{'olduser'} = $_[1]->{'user'};
			$uinfo->{'user'} = $_[0]->{'user'};
			&rename_mail_file($uinfo, $_[1]->{'user'});
			}

		if ($_[0]->{'home'} ne $_[1]->{'home'}) {
			# Home directory was changed
			$uinfo->{'home'} = $_[0]->{'home'};
			}

		&modify_user($uinfo, \%old, undef);
		&$second_print($text{'setup_done'});
		}
	&useradmin::unlock_user_files();

	if ($config{'home_quotas'} && $access{'edit'} == 1) {
		# Update his quotas 
		&$first_print($text{'save_quota'});
		&set_server_quotas($_[0]);
		&$second_print($text{'setup_done'});
		}
	}
}

# delete_unix(&domain)
# Delete the unix user and group for a domain
sub delete_unix
{
&require_useradmin();
local @allusers = &useradmin::list_users();
local ($uinfo) = grep { $_->{'user'} eq $_[0]->{'user'} } @allusers;

if (!$_[0]->{'parent'}) {
	# Delete unix user
	&useradmin::lock_user_files();
	if ($uinfo) {
		&$first_print($text{'delete_user'});
		&delete_user($uinfo);
		&$second_print($text{'setup_done'});
		}

	# Delete unix group
	local @allgroups = &useradmin::list_groups();
	local ($ginfo) = grep { $_->{'group'} eq $_[0]->{'group'} } @allgroups;
	if ($ginfo) {
		&$first_print($text{'delete_group'});
		&useradmin::set_group_envs($ginfo, 'DELETE_GROUP');
		&useradmin::making_changes();
		&useradmin::delete_group($ginfo);
		&useradmin::made_changes();
		&$second_print($text{'setup_done'});
		}
	&useradmin::unlock_user_files();
	}
}

# check_unix_clash(&domain, [field])
sub check_unix_clash
{
return 0 if ($_[0]->{'parent'});	# user already exists!
if (!$_[1] || $_[1] eq 'user') {
	return 1 if (defined(getpwnam($_[0]->{'user'})));
	}
if (!$_[1] || $_[1] eq 'group') {
	return 1 if (defined(getgrnam($_[0]->{'group'})));
	}
return 0;
}

# disable_unix(&domain)
# Lock out the password of this domain's Unix user
sub disable_unix
{
if (!$_[0]->{'parent'}) {
	&require_useradmin();
	&useradmin::lock_user_files();
	local @allusers = &useradmin::list_users();
	local ($uinfo) = grep { $_->{'user'} eq $_[0]->{'user'} } @allusers;
	if ($uinfo) {
		&$first_print($text{'disable_unix'});
		&useradmin::set_user_envs($uinfo, 'MODIFY_USER', "");
		&useradmin::making_changes();
		$_[0]->{'disabled_oldpass'} = $uinfo->{'pass'};
		$uinfo->{'pass'} = $uconfig{'lock_string'};
		&useradmin::modify_user($uinfo, $uinfo);
		&useradmin::made_changes();
		&$second_print($text{'setup_done'});
		}
	&useradmin::unlock_user_files();
	}
}

# enable_unix(&domain)
# Re-enable this domain's Unix user
sub enable_unix
{
if (!$_[0]->{'parent'}) {
	&require_useradmin();
	&useradmin::lock_user_files();
	local @allusers = &useradmin::list_users();
	local ($uinfo) = grep { $_->{'user'} eq $_[0]->{'user'} } @allusers;
	if ($uinfo) {
		&$first_print($text{'enable_unix'});
		&useradmin::set_user_envs($uinfo, 'MODIFY_USER', "");
		&useradmin::making_changes();
		$uinfo->{'pass'} = $_[0]->{'disabled_oldpass'};
		delete($_[0]->{'disabled_oldpass'});
		&useradmin::modify_user($uinfo, $uinfo);
		&useradmin::made_changes();
		&$second_print($text{'setup_done'});
		}
	&useradmin::unlock_user_files();
	}
}

# backup_unix(&domain, file)
# Does nothing, since the domain's unix user details are already known
sub backup_unix
{
return 1;
}

# restore_unix(&domain, file, &options)
# Extracts the given tar file into a user's home directory
sub restore_unix
{
&$first_print($text{'restore_unixuser'});

# Also re-set quotas
if ($config{'home_quotas'}) {
	&set_server_quotas($_[0]);
	}

# And update password and description
&require_useradmin();
local @allusers = &useradmin::list_users();
local ($uinfo) = grep { $_->{'user'} eq $_[0]->{'user'} } @allusers;
if ($uinfo && !$_[0]->{'parent'}) {
	&useradmin::lock_user_files();
	$uinfo->{'real'} = $_[0]->{'owner'};
	local $enc = &useradmin::encrypt_password(
			$_[0]->{'pass'});
	$uinfo->{'pass'} = $enc;
	&set_pass_change($uinfo);
	&useradmin::set_user_envs(
		$uinfo, 'MODIFY_USER', $_[0]->{'pass'});
	&useradmin::making_changes();
	&useradmin::modify_user($uinfo, $uinfo);
	&useradmin::made_changes();
	&useradmin::lock_user_files();
	}
&$second_print($text{'setup_done'});

return 1;
}

1;

